//
//  AEK2Overlay.m
//  Apple Extended Keyboard II Overlay Generator
//
//  Created by Peter Hosey on 2008-05-03.
//  Copyright 2008 Peter Hosey. All rights reserved.
//

#import "AEK2Overlay.h"

#import "PRHLineEnumerator.h"

@implementation AEK2Overlay

- init {
	if((self = [super init])) {
		EPSData = [[NSData alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AEK2-FunctionKeys" ofType:@"eps"]];

		authorName = [NSFullUserName() copy];
	}
	return self;
}
- (void) dealloc {
	[EPSData release];

	[programOverlayIsFor release];
	[authorName          release];

	[labelF1  release];
	[labelF2  release];
	[labelF3  release];
	[labelF4  release];
	[labelF5  release];
	[labelF6  release];
	[labelF7  release];
	[labelF8  release];
	[labelF9  release];
	[labelF10 release];
	[labelF11 release];
	[labelF12 release];
	[labelF13 release];
	[labelF14 release];
	[labelF15 release];
	[super dealloc];
}

#pragma mark ISO 8859-1

- (NSData *) isoLatin1DataForString:(NSString *)string {
	return [string dataUsingEncoding:NSISOLatin1StringEncoding allowLossyConversion:YES];
}

#pragma mark Accessors

- (NSString *) programOverlayIsFor {
	return programOverlayIsFor;
}
- (void) setProgramOverlayIsFor:(NSString *)newProgramOverlayIsFor {
	if(programOverlayIsFor != newProgramOverlayIsFor) {
		[programOverlayIsFor release];
		programOverlayIsFor = [newProgramOverlayIsFor copy];
	}
}

- (NSString *) authorName {
	return authorName;
}
- (void) setAuthorName:(NSString *)newAuthorName {
	if(authorName != newAuthorName) {
		[authorName release];
		authorName = [newAuthorName copy];
	}
}

#pragma mark -

- (NSString *) labelF1 {
	return labelF1;
}
- (BOOL) validateLabelF1:(NSString *)newLabelF1 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF1];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F1 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF1:(NSString *)newLabelF1 {
	if(labelF1 != newLabelF1) {
		[labelF1 release];
		labelF1 = [newLabelF1 copy];
	}
}

- (NSString *) labelF2 {
	return labelF2;
}
- (BOOL) validateLabelF2:(NSString *)newLabelF2 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF2];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F2 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF2:(NSString *)newLabelF2 {
	if(labelF2 != newLabelF2) {
		[labelF2 release];
		labelF2 = [newLabelF2 copy];
	}
}

- (NSString *) labelF3 {
	return labelF3;
}
- (BOOL) validateLabelF3:(NSString *)newLabelF3 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF3];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F3 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF3:(NSString *)newLabelF3 {
	if(labelF3 != newLabelF3) {
		[labelF3 release];
		labelF3 = [newLabelF3 copy];
	}
}

- (NSString *) labelF4 {
	return labelF4;
}
- (BOOL) validateLabelF4:(NSString *)newLabelF4 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF4];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F4 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF4:(NSString *)newLabelF4 {
	if(labelF4 != newLabelF4) {
		[labelF4 release];
		labelF4 = [newLabelF4 copy];
	}
}

- (NSString *) labelF5 {
	return labelF5;
}
- (BOOL) validateLabelF5:(NSString *)newLabelF5 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF5];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F5 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF5:(NSString *)newLabelF5 {
	if(labelF5 != newLabelF5) {
		[labelF5 release];
		labelF5 = [newLabelF5 copy];
	}
}

- (NSString *) labelF6 {
	return labelF6;
}
- (BOOL) validateLabelF6:(NSString *)newLabelF6 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF6];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F6 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF6:(NSString *)newLabelF6 {
	if(labelF6 != newLabelF6) {
		[labelF6 release];
		labelF6 = [newLabelF6 copy];
	}
}

- (NSString *) labelF7 {
	return labelF7;
}
- (BOOL) validateLabelF7:(NSString *)newLabelF7 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF7];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F7 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF7:(NSString *)newLabelF7 {
	if(labelF7 != newLabelF7) {
		[labelF7 release];
		labelF7 = [newLabelF7 copy];
	}
}

- (NSString *) labelF8 {
	return labelF8;
}
- (BOOL) validateLabelF8:(NSString *)newLabelF8 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF8];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F8 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF8:(NSString *)newLabelF8 {
	if(labelF8 != newLabelF8) {
		[labelF8 release];
		labelF8 = [newLabelF8 copy];
	}
}

- (NSString *) labelF9 {
	return labelF9;
}
- (BOOL) validateLabelF9:(NSString *)newLabelF9 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF9];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F9 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF9:(NSString *)newLabelF9 {
	if(labelF9 != newLabelF9) {
		[labelF9 release];
		labelF9 = [newLabelF9 copy];
	}
}

- (NSString *) labelF10 {
	return labelF10;
}
- (BOOL) validateLabelF10:(NSString *)newLabelF10 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF10];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F10 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF10:(NSString *)newLabelF10 {
	if(labelF10 != newLabelF10) {
		[labelF10 release];
		labelF10 = [newLabelF10 copy];
	}
}

- (NSString *) labelF11 {
	return labelF11;
}
- (BOOL) validateLabelF11:(NSString *)newLabelF11 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF11];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F11 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF11:(NSString *)newLabelF11 {
	if(labelF11 != newLabelF11) {
		[labelF11 release];
		labelF11 = [newLabelF11 copy];
	}
}

- (NSString *) labelF12 {
	return labelF12;
}
- (BOOL) validateLabelF12:(NSString *)newLabelF12 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF12];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F12 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF12:(NSString *)newLabelF12 {
	if(labelF12 != newLabelF12) {
		[labelF12 release];
		labelF12 = [newLabelF12 copy];
	}
}

- (NSString *) labelF13 {
	return labelF13;
}
- (BOOL) validateLabelF13:(NSString *)newLabelF13 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF13];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F13 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF13:(NSString *)newLabelF13 {
	if(labelF13 != newLabelF13) {
		[labelF13 release];
		labelF13 = [newLabelF13 copy];
	}
}

- (NSString *) labelF14 {
	return labelF14;
}
- (BOOL) validateLabelF14:(NSString *)newLabelF14 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF14];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F14 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF14:(NSString *)newLabelF14 {
	if(labelF14 != newLabelF14) {
		[labelF14 release];
		labelF14 = [newLabelF14 copy];
	}
}

- (NSString *) labelF15 {
	return labelF15;
}
- (BOOL) validateLabelF15:(NSString *)newLabelF15 error:(out NSError **)outError {
	NSData *isoLatin1Data = [self isoLatin1DataForString:newLabelF15];

	if (outError && !isoLatin1Data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Couldn't convert F15 label to ISO 8859-1", /*comment*/ @"Error descriptions"), NSLocalizedDescriptionKey,
			NSLocalizedString(@"The label contains one or more characters that ISO 8859-1 cannot even approximate.", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			NSLocalizedString(@"Try removing any unusual punctuation from the label text.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
			[NSNumber numberWithInt:NSISOLatin1StringEncoding], NSStringEncodingErrorKey,
			nil];
		*outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSKeyValueValidationError userInfo:userInfo];
	}

	return (isoLatin1Data != nil);
}
- (void) setLabelF15:(NSString *)newLabelF15 {
	if(labelF15 != newLabelF15) {
		[labelF15 release];
		labelF15 = [newLabelF15 copy];
	}
}

#pragma mark Generating the filled-in EPS document

- (NSData *) filledInEPSData {
	NSString *EPSString = [[[NSString alloc] initWithData:EPSData encoding:NSISOLatin1StringEncoding] autorelease];
	NSMutableString *filledInEPSString = [NSMutableString string];
	PRHLineEnumerator *linesEnum = [EPSString lineEnumerator];
	NSString *line;
	while ((line = [linesEnum nextObject])) {
		NSMutableString *mutableLine = [[line mutableCopy] autorelease];

		[mutableLine replaceOccurrencesOfString:@"$$PROGRAM_NAME$$" withString:(programOverlayIsFor ? programOverlayIsFor : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$AUTHOR_NAME$$" withString:(authorName ? authorName : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];

		[mutableLine replaceOccurrencesOfString:@"$$F1_LABEL$$" withString:(labelF1 ? labelF1 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F2_LABEL$$" withString:(labelF2 ? labelF2 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F3_LABEL$$" withString:(labelF3 ? labelF3 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F4_LABEL$$" withString:(labelF4 ? labelF4 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F5_LABEL$$" withString:(labelF5 ? labelF5 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F6_LABEL$$" withString:(labelF6 ? labelF6 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F7_LABEL$$" withString:(labelF7 ? labelF7 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F8_LABEL$$" withString:(labelF8 ? labelF8 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F9_LABEL$$" withString:(labelF9 ? labelF9 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F10_LABEL$$" withString:(labelF10 ? labelF10 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F11_LABEL$$" withString:(labelF11 ? labelF11 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F12_LABEL$$" withString:(labelF12 ? labelF12 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F13_LABEL$$" withString:(labelF13 ? labelF13 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F14_LABEL$$" withString:(labelF14 ? labelF14 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];
		[mutableLine replaceOccurrencesOfString:@"$$F15_LABEL$$" withString:(labelF15 ? labelF15 : @"") options:NSLiteralSearch range:(NSRange){ 0U, [mutableLine length] }];

		[filledInEPSString appendString:mutableLine];
	}

	return [self isoLatin1DataForString:filledInEPSString];
}

@end
