//
//  AEK2OverlayDocument.m
//  Apple Extended Keyboard II Overlay Generator
//
//  Created by Peter Hosey on 2008-05-03.
//  Copyright 2008 Peter Hosey. All rights reserved.
//

#import <Quartz/Quartz.h>
#import "NSObject+MapKVCKeysToDictionaryKeys.h"

#import "AEK2OverlayDocument.h"

#import "AEK2Overlay.h"
#import "TemplateKeys.h"

//In Mac OS X 10.4.10, these constants are defined in PDFDocument.h, but they don't actually exist in the framework.
//In Mac OS X 10.5.2, they do exist in the framework.
#if MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_4
#define PDFDocumentTitleAttribute  	 (NSString *)kCGPDFContextTitle
#define PDFDocumentAuthorAttribute   (NSString *)kCGPDFContextAuthor
#define PDFDocumentCreatorAttribute  (NSString *)kCGPDFContextCreator
#define PDFDocumentKeywordsAttribute @"Keywords" 
#endif

@implementation AEK2OverlayDocument

- init {
	if((self = [super init])) {
		overlay = [[AEK2Overlay alloc] init];

		[overlay addObserver:self forKeyPath:@"programOverlayIsFor" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"authorName" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF1"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF2"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF3"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF4"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF5"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF6"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF7"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF8"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF9"  options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF10" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF11" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF12" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF13" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF14" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];
		[overlay addObserver:self forKeyPath:@"labelF15" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:NULL];

		//Default orientation is landscape.
		NSPrintInfo *defaultPrintInfo = [self printInfo];
		[defaultPrintInfo setOrientation:NSLandscapeOrientation];
		[defaultPrintInfo   setLeftMargin:0.0f];
		[defaultPrintInfo    setTopMargin:0.0f];
		[defaultPrintInfo setBottomMargin:0.0f];
		[defaultPrintInfo  setRightMargin:0.0f];
	}
	return self;
}
- (void) dealloc {
	[overlay release];
	[imageView release];

	[super dealloc];
}

- (NSString *)windowNibName {
	return @"OverlayGenerator";
}

#pragma mark NSWindow delegate conformance

//Without this method, the window has its own undo manager, separate from ours, which means that our undo registrations (in the KVO observation method) have no user-visible effect.
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)sender {
	return [self undoManager];
}

#pragma mark KVO observer conformance

- (SEL) setterSelectorForKey:(NSString *)key {
	/*1. Prepend “set”.
	 *2. Capitalize the first character in the key.
	 *3. Append “:”.
	 *4. Convert the string to a selector.
	 */
	return NSSelectorFromString([[@"set" stringByAppendingString:[[[key substringToIndex:1U] uppercaseString] stringByAppendingString:[key substringFromIndex:1U]]] stringByAppendingString:@":"]);
}
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:object change:(NSDictionary *)change context:(void *)context {
	if (object == overlay) {
		NSAssert([keyPath rangeOfString:@"."].location == NSNotFound, @"Can't register undo with a key path, only a single key");

		id oldValue = [change objectForKey:NSKeyValueChangeOldKey];
		if (oldValue == [NSNull null])
			oldValue = nil;

		NSUndoManager *undoManager = [self undoManager];
		[undoManager registerUndoWithTarget:object selector:[self setterSelectorForKey:keyPath] object:oldValue];
		[undoManager setActionName:NSLocalizedStringFromTable(keyPath, @"Key paths as user actions", /*comment*/ nil)];
	}
}

#pragma mark NSNotifications

- (void)windowWillClose:(NSNotification *)notification {
	//End our KVO registrations in preparation for -dealloc.
	[overlay removeObserver:self forKeyPath:@"programOverlayIsFor"];
	[overlay removeObserver:self forKeyPath:@"authorName"];
	[overlay removeObserver:self forKeyPath:@"labelF1"];
	[overlay removeObserver:self forKeyPath:@"labelF2"];
	[overlay removeObserver:self forKeyPath:@"labelF3"];
	[overlay removeObserver:self forKeyPath:@"labelF4"];
	[overlay removeObserver:self forKeyPath:@"labelF5"];
	[overlay removeObserver:self forKeyPath:@"labelF6"];
	[overlay removeObserver:self forKeyPath:@"labelF7"];
	[overlay removeObserver:self forKeyPath:@"labelF8"];
	[overlay removeObserver:self forKeyPath:@"labelF9"];
	[overlay removeObserver:self forKeyPath:@"labelF10"];
	[overlay removeObserver:self forKeyPath:@"labelF11"];
	[overlay removeObserver:self forKeyPath:@"labelF12"];
	[overlay removeObserver:self forKeyPath:@"labelF13"];
	[overlay removeObserver:self forKeyPath:@"labelF14"];
	[overlay removeObserver:self forKeyPath:@"labelF15"];
}

#pragma mark Accessors

- (NSFont *) keyCapFont {
	NSFont *font = nil;

	//First choice: Univers Std Condensed Oblique.
	font = [NSFont fontWithName:@"Univers-CondensedOblique" size:14.0f];
	if (!font) {
		//Second choice: Univers Std *Light* Condensed Oblique.
		font = [NSFont fontWithName:@"Univers-LightCondensedOblique" size:14.0f];
		if (!font) {
			//Third choice: Arial Narrow Italic.
			font = [NSFont fontWithName:@"ArialNarrow-Italic" size:14.0f];
			if (!font) {
				//Final fall-back: The system font.
				font = [NSFont systemFontOfSize:0.0f];
			}
		}
	}

	return font;
}

#pragma mark Output data creation

- (NSData *) EPSData_error:(out NSError **)outError {
	NSData *data;
	NSError *error = nil;

	data = [overlay filledInEPSData];
	if (!data) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Could not create PostScript program", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			nil];
		error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		goto fail;
	}


fail:
	if (outError)
		*outError = error;

	return data;
}
- (NSData *) PDFData_error:(out NSError **)outError {
	NSData *data = [self EPSData_error:outError];
	NSError *error = nil;
	if (!data)
		goto fail;

	struct CGPSConverterCallbacks callbacks = { 0U };
	CGPSConverterRef converter = CGPSConverterCreate(/*info*/ NULL, &callbacks, /*options*/ NULL);
	if (!converter) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Could not create PostScript-to-PDF converter", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			nil];
		error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		goto fail;
	}

	CGDataProviderRef provider = CGDataProviderCreateWithCFData((CFDataRef)data);
	if (!provider) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Could not create data provider for converting PostScript program to PDF document", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			nil];
		error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		goto fail;
	}

	NSMutableData *PDFData = [NSMutableData data];
	CGDataConsumerRef consumer = CGDataConsumerCreateWithCFData((CFMutableDataRef)PDFData);
	if (!consumer) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Could not create data consumer for converting PostScript program to PDF document", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			nil];
		error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		goto fail;
	}

	bool success = CGPSConverterConvert(converter, provider, consumer, /*options*/ NULL);
	if (!success) {
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
			NSLocalizedString(@"Could not convert PostScript program to PDF document", /*comment*/ @"Failure reasons"), NSLocalizedFailureReasonErrorKey,
			nil];
		error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		goto fail;
	}

	//Set the title and author name of the document.
	PDFDocument *document = [[[PDFDocument alloc] initWithData:PDFData] autorelease];
	//This step isn't strictly necessary, so we don't throw an error if it fails.
	if (document) {
		NSMutableDictionary *attrs = [[[document documentAttributes] mutableCopy] autorelease];
		if (attrs) {
			NSString *program = [overlay programOverlayIsFor];
			if (program && [program length] == 0U)
				program = nil;
			if (program) {
				NSString *title = [NSString stringWithFormat:NSLocalizedString(@"Apple Extended Keyboard II overlay for %@", /*comment*/ @"PDF metadata title"), program];
				[attrs setObject:title forKey:PDFDocumentTitleAttribute];
			}

			NSString *author = [overlay authorName];
			if (author && [author length]) {
				[attrs setObject:author forKey:PDFDocumentAuthorAttribute];
			}

			[attrs setObject:[[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleDisplayName"] forKey:PDFDocumentCreatorAttribute];

			NSArray *keywords = [NSArray arrayWithObjects:
				@"Apple Extended Keyboard II",
				@"AEK2",
				@"function keys",
				@"overlay",
				//Code-names.
				@"Elmer", @"Nimitz",
				//This should be last because it can be the nil sentinel.
				program,
				nil];
			[attrs setObject:keywords forKey:PDFDocumentKeywordsAttribute];

			[document setDocumentAttributes:attrs];
		}

		data = [document dataRepresentation];
	}

fail:
	if (outError)
		*outError = error;

	return data;
}

#pragma mark Actions

- (void) saveOverlayAsPDF:(BOOL)convertToPDF {
	//Commit any editing the user hasn't tabbed out of.
	[overlayController commitEditing];

	NSString *extension = convertToPDF ? @"pdf" : @"eps";
	NSSavePanel *savePanel = [NSSavePanel savePanel];
	[savePanel setRequiredFileType:extension];
	[savePanel setCanSelectHiddenExtension:YES];

	[savePanel beginSheetForDirectory:nil
								 file:[[[[self fileName] lastPathComponent] stringByDeletingPathExtension] stringByAppendingPathExtension:extension]
					   modalForWindow:[self windowForSheet]
						modalDelegate:self
					   didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
						  contextInfo:NULL];
}
- (void) savePanelDidEnd:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void  *)contextInfo {
	if (returnCode == NSOKButton) {
		NSError *error = nil;

		[self saveToURL:[savePanel URL]
				 ofType:[savePanel requiredFileType]
	   forSaveOperation:NSSaveToOperation
				  error:&error];

		if (error) {
			[self presentError:error
				modalForWindow:[self windowForSheet]
					  delegate:self
			didPresentSelector:@selector(didPresentErrorWithRecovery:contextInfo:)
				   contextInfo:NULL];
		}
	}
}

- (IBAction) saveEPSFileAs:sender {
	[self saveOverlayAsPDF:NO];
}
- (IBAction) savePDFFileAs:sender {
	[self saveOverlayAsPDF:YES];
}

#pragma mark NSDocument I/O methods

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError {
	enum { ROMAN_NUMERAL_TWO = 0x2161 };

	if ([typeName isEqualToString:[NSString stringWithFormat:@"Apple Extended Keyboard %C overlay template", ROMAN_NUMERAL_TWO]]
	|| UTTypeEqual((CFStringRef)typeName, (CFStringRef)@"org.boredzo.aek2-overlay-template")
	) {
		NSString *errorString = nil;
		NSDictionary *dict = [NSPropertyListSerialization propertyListFromData:data
															  mutabilityOption:NSPropertyListImmutable
																		format:&format
															  errorDescription:&errorString];
		if (!dict) {
			if (outError) {
				NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
					errorString, NSLocalizedFailureReasonErrorKey,
					NSLocalizedString(@"Contact whoever supplied you with the file to make sure you have an updated version.", /*comment*/ @"Recovery suggestions"), NSLocalizedRecoverySuggestionErrorKey,
					nil];
				NSError *error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileReadCorruptFileError userInfo:userInfo];
				*outError = error;
			}
		}

		//Disable undo registration, because we don't want the user to be able to undo what's already on disk.
		NSUndoManager *undoManager = [self undoManager];
		[undoManager disableUndoRegistration];

		NSString *newProgramName = [dict objectForKey:TEMPLATE_KEY_PROGRAM_NAME];
		[overlay setProgramOverlayIsFor:newProgramName];
		NSString *newAuthorName = [dict objectForKey:TEMPLATE_KEY_AUTHOR_NAME];
		if (!(newAuthorName && [newAuthorName length]))
			[overlay setAuthorName:NSFullUserName()];
		else
			[overlay setAuthorName:newAuthorName];

		NSDictionary *labels = [dict objectForKey:TEMPLATE_KEY_LABELS];
		[overlay setLabelF1: [labels objectForKey:TEMPLATE_LABEL_KEY_F1]];
		[overlay setLabelF2: [labels objectForKey:TEMPLATE_LABEL_KEY_F2]];
		[overlay setLabelF3: [labels objectForKey:TEMPLATE_LABEL_KEY_F3]];
		[overlay setLabelF4: [labels objectForKey:TEMPLATE_LABEL_KEY_F4]];
		[overlay setLabelF5: [labels objectForKey:TEMPLATE_LABEL_KEY_F5]];
		[overlay setLabelF6: [labels objectForKey:TEMPLATE_LABEL_KEY_F6]];
		[overlay setLabelF7: [labels objectForKey:TEMPLATE_LABEL_KEY_F7]];
		[overlay setLabelF8: [labels objectForKey:TEMPLATE_LABEL_KEY_F8]];
		[overlay setLabelF9: [labels objectForKey:TEMPLATE_LABEL_KEY_F9]];
		[overlay setLabelF10:[labels objectForKey:TEMPLATE_LABEL_KEY_F10]];
		[overlay setLabelF11:[labels objectForKey:TEMPLATE_LABEL_KEY_F11]];
		[overlay setLabelF12:[labels objectForKey:TEMPLATE_LABEL_KEY_F12]];
		[overlay setLabelF13:[labels objectForKey:TEMPLATE_LABEL_KEY_F13]];
		[overlay setLabelF14:[labels objectForKey:TEMPLATE_LABEL_KEY_F14]];
		[overlay setLabelF15:[labels objectForKey:TEMPLATE_LABEL_KEY_F15]];

		[undoManager enableUndoRegistration];

		return YES;
	}

	return NO;
}
- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError {
	NSData *data = nil;
	NSError *error = nil;

	BOOL convertToPDF = [typeName isEqualToString:@"pdf"] || UTTypeEqual((CFStringRef)typeName, (CFStringRef)@"com.adobe.pdf");
	BOOL generateEPS = convertToPDF || [typeName isEqualToString:@"eps"] || UTTypeEqual((CFStringRef)typeName, (CFStringRef)@"com.adobe.encapsulated-postscript");

	if (!generateEPS) {
		//We're saving the overlay-template document.
		NSDictionary *keyMapping = [NSDictionary dictionaryWithObjectsAndKeys:
			TEMPLATE_KEY_PROGRAM_NAME, @"programOverlayIsFor",
			TEMPLATE_KEY_AUTHOR_NAME,  @"authorName",
			nil];
		NSMutableDictionary *dict = [[[overlay valuesForKeysWithKeyMapping:keyMapping] mutableCopy] autorelease];
		NSDictionary *labelsKeyMapping = [NSDictionary dictionaryWithObjectsAndKeys:
			TEMPLATE_LABEL_KEY_F1,  @"labelF1",
			TEMPLATE_LABEL_KEY_F2,  @"labelF2",
			TEMPLATE_LABEL_KEY_F3,  @"labelF3",
			TEMPLATE_LABEL_KEY_F4,  @"labelF4",
			TEMPLATE_LABEL_KEY_F5,  @"labelF5",
			TEMPLATE_LABEL_KEY_F6,  @"labelF6",
			TEMPLATE_LABEL_KEY_F7,  @"labelF7",
			TEMPLATE_LABEL_KEY_F8,  @"labelF8",
			TEMPLATE_LABEL_KEY_F9,  @"labelF9",
			TEMPLATE_LABEL_KEY_F10, @"labelF10",
			TEMPLATE_LABEL_KEY_F11, @"labelF11",
			TEMPLATE_LABEL_KEY_F12, @"labelF12",
			TEMPLATE_LABEL_KEY_F13, @"labelF13",
			TEMPLATE_LABEL_KEY_F14, @"labelF14",
			TEMPLATE_LABEL_KEY_F15, @"labelF15",
			nil];
		[dict setObject:[overlay valuesForKeysWithKeyMapping:labelsKeyMapping] forKey:TEMPLATE_KEY_LABELS];

		NSString *errorString = nil;
		data = [NSPropertyListSerialization dataFromPropertyList:dict
														  format:format
												errorDescription:&errorString];
		if (!data) {
			NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
				errorString, NSLocalizedFailureReasonErrorKey,
				nil];
			error = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:userInfo];
		}
	} else {
		//We're exporting EPS or PDF data.
		data = convertToPDF ? [self PDFData_error:outError] : [self EPSData_error:outError];
		if (!data)
			goto fail;
	}

fail:
	if (outError)
		*outError = error;

	return data;
}
- (void)didPresentErrorWithRecovery:(BOOL)didRecover contextInfo:(void *)contextInfo {
	//We don't really care.
}

#pragma mark NSDocument printing methods

- (NSPrintOperation *) printOperationWithSettings:(NSDictionary *)printSettings error:(out NSError **)outError {
	NSPrintInfo *printInfo = [self printInfo];
	[[printInfo dictionary] setValuesForKeysWithDictionary:printSettings];
	NSPrintOperation *printOp = nil;

	NSData *pdfData = [self PDFData_error:outError];
	if (pdfData) {
		if (imageView)
			[imageView release];
		imageView = [[NSImageView alloc] initWithFrame:(NSRect){ NSZeroPoint, [printInfo paperSize] }];

		NSImage *image = [[[NSImage alloc] initWithData:pdfData] autorelease];
		[image setCacheMode:NSImageCacheNever];
		[image setDataRetained:YES];
		[imageView setImage:image];

		printOp = [NSPrintOperation printOperationWithView:imageView printInfo:printInfo];
	}

	return printOp;
}

@end
